@@ -1,6 +1,6 @@
 #############################################################################
 #  
-#	$Id: Clients.py,v 2.15 2005/01/22 01:05:43 irmen Exp $
+#	$Id: Clients.py,v 2.18 2007/03/07 21:49:54 irmen Exp $
 #	Event Service client base classes
 #
 #	This is part of "Pyro" - Python Remote Objects
@@ -23,23 +23,29 @@ class Subscriber(Pyro.core.CallbackObjBase):
 		self.NS = locator.getNS(host=Pyro.config.PYRO_NS_HOSTNAME)
 		daemon.useNameServer(self.NS)
 		daemon.connect(self)  #  will also set self.daemon...
-		uri = self.NS.resolve(Pyro.constants.EVENTSERVER_NAME)
-		self.eventservice=Pyro.core.getProxyForURI(uri)
-		self.eventservice._setIdentification(ident)
+		self.ES_uri = self.NS.resolve(Pyro.constants.EVENTSERVER_NAME)
+		self.ES_ident=ident
 		self.abortListen=0
 		self.daemon=daemon	# make sure daemon doesn't get garbage collected now
 
+	def getES(self):
+		# we get a fresh proxy to the ES because of threading issues.
+		# (proxies can not be reused across multiple threads)
+		eventservice=Pyro.core.getProxyForURI(self.ES_uri)
+		eventservice._setIdentification(self.ES_ident)
+		return eventservice
+
 	def subscribe(self,subjects):
 		# Subscribe to one or more subjects.
 		# It is safe to call this multiple times.
-		self.eventservice.subscribe(subjects, self.getProxy())
+		self.getES().subscribe(subjects, self.getProxy())
 	def subscribeMatch(self,subjectPatterns):
 		# Subscribe to one or more subjects (by pattern)
 		# It is safe to call this multiple times.
-		self.eventservice.subscribeMatch(subjectPatterns, self.getProxy())
+		self.getES().subscribeMatch(subjectPatterns, self.getProxy())
 	def unsubscribe(self, subjects):
 		# Unsubscribe the subscriber for the given subject(s).
-		self.eventservice.unsubscribe(subjects, self.getProxy())
+		self.getES().unsubscribe(subjects, self.getProxy())
 
 	def abort(self):
 		self.abortListen=1
@@ -59,10 +65,19 @@ class Publisher:
 		Pyro.core.initClient()
 		locator = Pyro.naming.NameServerLocator(identification=ident)
 		ns = locator.getNS(host=Pyro.config.PYRO_NS_HOSTNAME)
-		uri = ns.resolve(Pyro.constants.EVENTSERVER_NAME)
-		self.eventservice=Pyro.core.getProxyForURI(uri)
-		self.eventservice._setIdentification(ident)
+		self.ES_uri = ns.resolve(Pyro.constants.EVENTSERVER_NAME)
+		self.ES_ident = ident
+		ns._release()   # be very sure to release the socket
+
+	def getES(self):
+		# we get a fresh proxy to the ES because of threading issues.
+		# (proxies can not be reused across multiple threads)
+		eventservice=Pyro.core.getProxyForURI(self.ES_uri)
+		eventservice._setIdentification(self.ES_ident)
+		return eventservice
 
 	def publish(self, subjects, msg):
-		self.eventservice.publish(subjects,msg)
+		es=self.getES()
+		es.publish(subjects,msg)
+		es._release()   # be very sure to release the socket
 
