@@ -3,23 +3,21 @@ import sys, os
 
 from Pyro.errors import *
 import Pyro.core
+import Pyro.util
 
 sys.path.insert(0,os.pardir)	# to find testclient.py
 
 import testclient
 
-import agent.ShoppingAgent as SA
+import agent.ShoppingAgent
 
 Pyro.config.PYRO_MOBILE_CODE=1		# Enable mobile code
 
 # Get a proxy with attrs 
 mall = testclient.getproxy('Shop1',True)
 
-import sys, os, string
-import Pyro.core
-
-Harry = SA.ShoppingAgent('Harry')
-Joyce = SA.ShoppingAgent('Joyce')
+Harry = agent.ShoppingAgent.ShoppingAgent('Harry')
+Joyce = agent.ShoppingAgent.ShoppingAgent('Joyce')
 
 try:
 	print 'Harry goes shopping...'
@@ -31,5 +29,4 @@ try:
 	Joyce.result()
 	print
 except Exception,x:
-	import Pyro.util
 	print ''.join(Pyro.util.getPyroTraceback(x))
