@@ -15,7 +15,7 @@ print 'Naming Service found at',ns.URI.address,'port',ns.URI.port
 
 print 'binding to object'
 try:
-    URI=ns.resolve('authentication')
+    URI=ns.resolve(':test.authentication')
     print 'URI:',URI
 except Pyro.core.PyroError,x:
     print 'Couldn\'t bind object, nameserver says:',x
