@@ -13,7 +13,7 @@ ns = locator.getNS()
 
 print 'Naming Service found at',ns.URI.address,'('+(Pyro.protocol.getHostname(ns.URI.address) or '??')+') port',ns.URI.port
 
-URI=ns.resolve('autoreconnect')
+URI=ns.resolve(':test.autoreconnect')
 obj = Pyro.core.getAttrProxyForURI(URI)
 
 while 1:
@@ -22,11 +22,11 @@ while 1:
 		obj.method(42)
 		print 'Sleeping 1 second'
 		time.sleep(1)
-		#obj._release() # XXX remove this!
+		#obj._release() # experiment with this
 		#print 'released'
-		time.sleep(2)
-	except Pyro.errors.ConnectionClosedError,x:
+		#time.sleep(2)
+	except Pyro.errors.ConnectionClosedError,x:     # or possibly even ProtocolError
 		print 'Connection lost. REBINDING...'
 		print '(restart the server now)'
-		obj.adapter.rebindURI()
+		obj.adapter.rebindURI(tries=10)
 
