@@ -1,4 +1,5 @@
 import Pyro.core
+import Pyro.util
 
 # a message bouncer. Passes messages back to the callback
 # object, until a certain limit is reached.
@@ -8,6 +9,7 @@ class Bouncer(Pyro.core.CallbackObjBase):
 		Pyro.core.ObjBase.__init__(self)
 		self.name=name
 		self.count=0
+		self.callbackMutex=Pyro.util.getLockObject()
 	def register(self, callback):
 		self.callback=callback
 	def process(self,message):
@@ -18,7 +20,14 @@ class Bouncer(Pyro.core.CallbackObjBase):
 
 		print "I'm",self.name,", bouncing back..."
 		message.append(self.name)
-		result=self.callback.process(message)
+		self.callbackMutex.acquire()
+		try:
+			# claim the callback proxy for ourselves.
+			# we can do this now because we are in a thread mutex.
+			self.callback._transferThread()
+			result=self.callback.process(message)
+		finally:
+			self.callbackMutex.release()
 		self.count+=1
 		result.insert(0,"passed on from "+self.name+':'+str(self.count))
 		print 'returned from callback'
