@@ -5,7 +5,8 @@ sys.path.insert(0,os.pardir)	# to find testclient.py
 
 import testclient
 
-obj = testclient.start(None,None,'hugetransfer')
+obj = testclient.getproxy('hugetransfer')
+# obj._setTimeout(2)
 
 import time
 
@@ -17,11 +18,11 @@ totalsize=0
 begin=time.time()
 for i in range(1,15):
 	print 'transferring',basesize*i,'bytes'
-	obj.transfer(data*i)
+	size=obj.transfer(data*i)
+	# print " reply=",size
 	totalsize=totalsize+basesize*i
 duration=time.time()-begin
 
 print 'It took',duration,'seconds to transfer',totalsize/1024,'kilobyte.'
 print 'That is',totalsize/1024/duration,'k/sec. = ',totalsize/1024/1024/duration,'mb/sec.'
 
-
