@@ -5,7 +5,7 @@ sys.path.insert(0,os.pardir)	# to find testclient.py
 
 import testclient
 
-test = testclient.start('inherit_proxy', 'Fsub', 'inheritance')
+test = testclient.getproxy('inheritance')
 
 print "base1.meth1? -->",test.meth1()		# should be base1.meth1
 print "Fsub.meth2?  -->",test.meth2()		# should be Fsub.meth2
