@@ -1,6 +1,7 @@
 #! /usr/bin/env python
 import Pyro.core
 import Pyro.naming
+from Pyro.errors import NamingError
 import tst
 
 # The testclass object.
@@ -20,7 +21,11 @@ ns=Pyro.naming.NameServerLocator().getNS()
 daemon=Pyro.core.Daemon()
 daemon.useNameServer(ns)
 
-uri=daemon.connect(testclass(),"simple")
+try:
+    ns.createGroup(":test")
+except NamingError:
+    pass
+uri=daemon.connect(testclass(),":test.simple")
 
 print "Server is ready."
 daemon.requestLoop()
