@@ -1,6 +1,6 @@
 from Pyro.protocol import DefaultConnValidator
 import Pyro.constants
-import md5
+import md5, hmac
 
 
 # Example username/password database:
@@ -12,19 +12,19 @@ EXAMPLE_ALLOWED_USERS = {
 
 #
 #	Example login/password validator.
-#	Passwords are munged using md5 with challenge,
-#	so they cannot be spied upon.
+#	Passwords are protected using md5 so they are not stored in plaintext.
+#	The actual identification check is done using a hmac-md5 secure hash.
 #
 class UserLoginConnValidator(DefaultConnValidator):
 
 	def _xform(self, s):
-		return md5.md5(s).digest()
+		return md5.new(s).digest()	# use md5 hash instead of plaintext
 
 	def acceptIdentification(self, daemon, connection, token, challenge):
 		login, password = token.split(':', 1)
 		realpass = EXAMPLE_ALLOWED_USERS.get(login)
 		# Check if the username/password is valid.
-		if realpass and self._xform(self._xform(realpass)+challenge) == password:
+		if realpass and hmac.new(challenge,self._xform(realpass)).digest() == password:
 			print "ALLOWED", login
 			connection.authenticated=login  # store for later reference by Pyro object
 			return(1,0)
@@ -32,7 +32,7 @@ class UserLoginConnValidator(DefaultConnValidator):
 		return (0,Pyro.constants.DENIED_SECURITY)
 		
 	def createAuthToken(self, authid, challenge, peeraddr, URI, daemon):
-		return "%s:%s" % (authid[0], self._xform(authid[1]+challenge))
+		return "%s:%s" % (authid[0], hmac.new(challenge,authid[1]).digest() )
 
 	def mungeIdent(self, ident):
 		# ident is tuple (login, password), the client sets this.
